import java.util.Random;

/**
 * Definicion de la clase AyudaCalculos. Ayuda a calular
 * 
 * @author Sebastian Dieguez
 */
public class AyudaCalculos {

	/**
	 * Obtiene un porcentaje de chance de golpear. Todos parten de una base de 50%. Si un dado de 1
	 * a 100 saca mas de este numero, deberia ERRAR.
	 * 
	 * @param destrezaAta
	 *            Destreza del atacante
	 * @param agilidadObj
	 *            Agilidad del objetivo del ataque
	 * @return el porcentaje que tengo de pegarle
	 */
	public static int chanceDeGolpe(int destrezaAta, int agilidadObj) {
		int res = 0;
		// Todos tienen un 50% de chance de pegar, a eso se suma la destreza
		res = 50 + destrezaAta;
		// Restamos la agilidad del objetivo
		res -= agilidadObj;

		return res;
	}

	/**
	 * @see AyudaCalculos#chanceDeGolpe(int, int)
	 */
	public static int chanceDeGolpe(Personaje atacante, Personaje objetivo) {
		return chanceDeGolpe(atacante.MOD_DESTREZA, objetivo.MOD_AGILIDAD);
	}

	/**
	 * Obtiene la cantidad de danio que este personaje debe inflijir.
	 * 
	 * @param danioArmaAta
	 *            Danio que inflinje el arma del atacante
	 * @param danioFuerzaAta
	 *            Danio que inflinje el atacante por su fuerza
	 * @param armaduraObj
	 *            Armadura que tiene el objetivo
	 * @param distancia
	 *            Distancia en mosaicos entre el objetivo y el atacante (minimo 1)
	 * @return Los puntos de danio que seran restados de la salud del objetivo
	 */
	public static int danioDeGolpe(int danioArmaAta, int danioFuerzaAta, int armaduraObj, int distancia) {
		int danio = 0;
		danio = (danioArmaAta + danioFuerzaAta) - armaduraObj;
		// Si la distancia es mayor que 1 (cuerpo a cuerpo) se aplica modificador para armas de
		// rango
		if (distancia > 1) {
			danio = (int) Math.ceil(danio / (distancia * 0.75));
		}
		return danio;
	}

	/**
	 * @see AyudaCalculos#danioDeGolpe(int, int, int, int)
	 */
	public static int danioDeGolpe(Personaje atacante, Personaje objetivo, int distancia) {
		return danioDeGolpe(atacante.MOD_ARMA, atacante.MOD_FUERZA, objetivo.MOD_ARMADURA, distancia);
	}

	/**
	 * @see AyudaCalculos#danioDeGolpe(int, int, int, int)
	 */
	public static int danioDeGolpe(Personaje atacante, Personaje objetivo) {
		return danioDeGolpe(atacante.MOD_ARMA, atacante.MOD_FUERZA, objetivo.MOD_ARMADURA, 1);
	}

	/**
	 * Tira un dado de un tamanio indicado
	 * 
	 * @param tamanio
	 *            cantidad de caras del dado
	 * @return un numero entre 1 y esas caras
	 */
	public static int tirarDado(int tamanio) {
		Random r = new Random();
		int dado = r.nextInt(tamanio) + 1;

		return dado;
	}

	public static int tirarD6() {
		return tirarDado(6);
	}

	public static int tirarD10() {
		return tirarDado(10);
	}

	public static int tirarD100() {
		return tirarDado(100);
	}

	public static boolean tirarDadoMayorQue(int tamanio, int minimo) {
		return (tirarDado(tamanio) > minimo);
	}

	public static boolean tirarDadoMenorQue(int tamanio, int maximo) {
		return (tirarDado(tamanio) < maximo);
	}
}
