import java.util.ArrayList;

/**
 * Un camino determinado por un algoritmo de busqueda. Una serie de puntos desde la posicion inicial hasta la final. Esto include el punto de la posicion inicial.
 * 
 * @author Sebastian Dieguez
 */
public class Camino {
	/** Una lista de Puntos que hacen a este camino */
	private ArrayList<Punto> puntos = new ArrayList<Punto>();

	/**
	 * Crea un camino vacio
	 */
	public Camino() {

	}

	/**
	 * Obtiene el largo del camino, o sea la cantidad de puntos
	 * 
	 * @return El numero de puntos de este camino
	 */
	public int getLargo() {
		return this.puntos.size();
	}

	/**
	 * Obtiene el punto dado un indice en el camino
	 * 
	 * @param indice El indice del punto a obtener. Debe ser >= 0 y < getLargo();
	 * @return La informacion del punto, una posicion en el mapa.
	 */
	public Punto getPunto(int indice) {
		return (Punto) this.puntos.get(indice);
	}

	/**
	 * Obtiene la coordenada x para el punto de ese indice
	 * 
	 * @param indice El indice del punto cuya coordenada x se quiere obtener
	 * @return La coordenada x para el punto de ese indice
	 */
	public int getX(int indice) {
		return this.getPunto(indice).getX();
	}

	/**
	 * Obtiene la coordenada y para el punto de ese indice
	 * 
	 * @param indice El indice del punto cuya coordenada y se quiere obtener
	 * @return La coordenada y para el punto de ese indice
	 */
	public int getY(int indice) {
		return this.getPunto(indice).getY();
	}

	/**
	 * Agrega un punto al camino
	 * 
	 * @param x La coordenada x del nuevo punto
	 * @param y La coordenada y del nuevo punto
	 */
	public void agregarPunto(int x, int y) {
		this.puntos.add(new Punto(x, y));
	}

	/**
	 * Antepone un punto al camino.
	 * 
	 * @param x La coordenada x del nuevo punto
	 * @param y La coordenada y del nuevo punto
	 */
	public void anteponerPunto(int x, int y) {
		this.puntos.add(0, new Punto(x, y));
	}

	/**
	 * Comprueba si el camino ya tiene este punto
	 * 
	 * @param x La coordenada x del punto a comprobar
	 * @param y La coordenada y del punto a comprobar
	 * @return TRUE si el camino tiene ese punto
	 */
	public boolean contiene(int x, int y) {
		return this.puntos.contains(new Punto(x, y));
	}
}
