import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

/**
 * Definicion de la clase EnanoMonje.
 * 
 * @author Sebastian Dieguez
 */
public class EnanoMonje extends BuenosMuchachos {

	int MOD_SANACION = 3; // Poder de sanacion de este monje

	/**
	 * Caso especial donde el monje se siente atraido por sus pacientes para sanarlos
	 */
	Personaje pacienteObjetivo = null;

	/**
	 * Construye un EnanoMonje
	 */
	public EnanoMonje() {
		this.nombre = "Enano Monje";

		// Caracteristicas
		this.MOD_FUERZA = 10;// Fuerza nula
		this.MOD_ARMA = 10;// Virote de fuego
		this.MOD_DESTREZA = 40;// Es muy diestro

		this.MOD_AGILIDAD = 40;// Es muy agil para esquivar
		this.MOD_ARMADURA = 0;// Sin Armadura
		this.MOD_RANGO = 3;// Rango de sanacion y disparo
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see BuenosMuchachos#getPjObjetivo()
	 */
	@Override
	public Personaje getPjObjetivo() {
		// Si no tenemos paciente o esta muerto
		if ((this.pacienteObjetivo == null) || (this.pacienteObjetivo.SALUD <= 0)) {
			this.obtenerPaciente();
		}
		// Si no encontramos ningun paciente, retornamos el enemigo
		if (this.pacienteObjetivo == null) {
			return this.pjObjetivo;
		}
		return this.pacienteObjetivo;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Personaje#setPjObjetivo(Personaje)
	 */
	@Override
	public void setPjObjetivo(Personaje objetivo) {
		// Si asignamos null de una que sea el paciente
		if (objetivo == null) {
			if (this.pacienteObjetivo != null) {
				this.pacienteObjetivo = null;
			} else {
				this.pjObjetivo = null;
			}
		} else {
			if (this.esAmigo(objetivo)) {
				this.pacienteObjetivo = objetivo;
			} else {
				this.pjObjetivo = objetivo;
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Personaje#ejecutarAcciones()
	 */
	@Override
	public void ejecutarAcciones() {
		super.ejecutarAcciones();
		if (this.estado == 2) {// Si podemos sanar mejor
			List<BuenosMuchachos> pacientes = this.obtenerPacientesPorRango();

			// Si hay pacientes cerca
			if (!pacientes.isEmpty()) {
				for (BuenosMuchachos bm : pacientes) {
					// Sanamos y mostramos en el paciente la curacion
					bm.SALUD += this.MOD_SANACION;
					this.getWorld().addObject(new EtiquetaPluma("+" + this.MOD_SANACION, Color.GREEN, Color.BLACK), bm.getX(), bm.getY());
				}
				// Si no hay pacientes cerca deberiamos caminar hacia un paciente
			} else {
				this.getPjObjetivo();
				this.estado = 1;
			}
		}
	}

	/**
	 * Obtiene los pacientes que estan en rango
	 * 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private List<BuenosMuchachos> obtenerPacientesPorRango() {
		List<BuenosMuchachos> lista = new ArrayList<BuenosMuchachos>();
		lista.addAll(this.getNeighbours(this.MOD_RANGO, true, EnanoGuerrero.class));
		lista.addAll(this.getNeighbours(this.MOD_RANGO, true, EnanoArquero.class));
		lista.addAll(this.getNeighbours(this.MOD_RANGO, true, EnanoMonje.class));

		List<BuenosMuchachos> res = new ArrayList<BuenosMuchachos>();

		for (BuenosMuchachos bm : lista) {
			if (bm.SALUD > 0) {
				res.add(bm);
			}
		}

		return res;
	}

	/**
	 * Obtiene un enano paciente, para ubicarse cerca y sanar
	 */
	private void obtenerPaciente() {
		// buscamos al amigo que tenga menos salud
		WarInTheDeepGame game = (WarInTheDeepGame) this.getWorld();
		List<Personaje> lista = game.obtenerBuenosMuchachos(true);
		Personaje minPj = null;
		int minSalud = Integer.MAX_VALUE;
		for (Personaje pj : lista) {
			if ((pj.SALUD < minSalud) && (pj.SALUD > 0) && (!this.equals(pj))) {
				minSalud = pj.SALUD;
				minPj = pj;
			}
		}

		this.pacienteObjetivo = minPj;
	}
}
