import greenfoot.Actor;

import java.awt.Color;
import java.util.List;

/**
 * @author SebaSOFT
 */
public abstract class Personaje extends Actor {

	public int SALUD = 80;

	public int MOD_FUERZA = 20;
	public int MOD_AGILIDAD = 5;
	public int MOD_DESTREZA = 20;
	public int MOD_ARMADURA = 20;
	public int MOD_ARMA = 32;
	public int MOD_RANGO = 1;
	public int MOD_VELOCIDAD = 1;
	public boolean GANADOR = false;

	public String nombre = "Personaje";

	protected Personaje pjObjetivo = null;

	protected Camino caminoObjetivo = null;

	public int estado = 1;// Comienzan caminando

	/**
	 * Procesa el estado y decide que hacer, este metodo se puede sobreescribir para funcionalidades especiales (ejemplo el monje)
	 */
	public void procesarEstado() {

		// Si alguien mas mato a nuestro enemigo
		if ((this.pjObjetivo != null) && (this.pjObjetivo.SALUD <= 0)) {
			this.pjObjetivo.SALUD = 0;
			this.pjObjetivo.GANADOR = false;
			this.pjObjetivo.muerte();
			this.pjObjetivo = null; // No tenemos objetivo
			this.caminoObjetivo = null; // Tenemos que buscar un camino
			this.estado = 1;// vamos a caminar
		}

		// si tenemos alguien en la mira
		if (this.pjObjetivo == null) {
			// buscamos objetivos en rango de tiro
			List<Personaje> malos = this.obtenerEnemigosEnRango();

			// si no hay objetivos en rango
			if ((malos == null) || (malos.size() == 0)) {
				// buscar dentro de los enemigos vivos
				malos = this.obtenerEnemigosVivos();
				this.estado = 1;// Caminar
			} else {
				this.estado = 2;// Atacar
			}

			// Si hay otro enemigo vivo
			if ((malos != null) && (malos.size() > 0)) {
				// Elijo un objetivo al azar
				this.pjObjetivo = malos.get(AyudaCalculos.tirarDado(malos.size()) - 1);
			} else {
				this.getWorld().addObject(new EtiquetaPluma("GANE!", Color.GREEN, Color.BLACK), this.getX(), this.getY());
				this.GANADOR = true;
			}
		}

		// Si conseguimos un enemigo
		if (this.getPjObjetivo() != null) {
			// Si ya estoy en rango de el objetivo, voy a atacar
			Punto origen = new Punto(this.getX(), this.getY());
			Punto destino = new Punto(this.getPjObjetivo().getX(), this.getPjObjetivo().getY());
			if (AyudaDistancia.distanciaEntre(origen, destino) <= this.MOD_RANGO) {
				this.estado = 2;// Atacar
			} else {
				this.estado = 1;// Caminar hacia el objetivo
			}
		} else { // No hay mas gente viva
			this.estado = 0;
		}

		// Si la salud es 0 nos morimos
		if (this.SALUD < 0) {
			this.SALUD = 0;
			this.muerte();
		}

	}

	/**
	 * Ejecuta las acciones (caminar, atacar, etc)
	 */
	public void ejecutarAcciones() {
		WarInTheDeepGame game = (WarInTheDeepGame) this.getWorld();

		switch (this.estado) {
			case 0: // Pasivo
				// No hacer nada?
				break;
			case 1: // Caminando
				this.accionCaminar(game);
				break;
			case 2: // Atacando
				this.accionAtacar(game);
				break;
		}
	}

	/**
	 * @param game
	 */
	protected int accionAtacar(WarInTheDeepGame game) {
		int res = 0;

		Personaje obj = this.getPjObjetivo();
		// Si hay personaje objetivo (en este caso puede ser amigo porque el monje retorna un paciente si exxiste)
		if (obj != null) {
			// Giramos hacia el personaje
			this.turnTowards(obj.getX(), obj.getY());
		}

		// Comprobamos si hay un enemigo realmente (pasa con los sanadores que tienen un objetivo
		// pero no un enemigo)
		if ((this.pjObjetivo != null) && this.esEnemigo(this.pjObjetivo)) {
			// HACER CALCULOS DE GOLPE
			int chance = AyudaCalculos.chanceDeGolpe(this, this.pjObjetivo);
			int dado100 = AyudaCalculos.tirarD100();
			if (dado100 < chance) {
				// Disparamos
				int danio = Math.abs(AyudaCalculos.danioDeGolpe(this, this.pjObjetivo, this.MOD_RANGO));

				// Si la chance igual a 1 el danio es critico
				if (dado100 == 1) {
					danio *= 2;
					game.addObject(new EtiquetaPluma("-" + danio + "!", Color.YELLOW, Color.RED), this.pjObjetivo.getX(), this.pjObjetivo.getY());
				} else {
					game.addObject(new EtiquetaPluma("-" + danio, Color.RED, Color.YELLOW), this.pjObjetivo.getX(), this.pjObjetivo.getY());
				}

				this.pjObjetivo.SALUD -= danio;

				res = danio;
			} else {
				// Fallamos
				game.addObject(new EtiquetaPluma("FALLE", Color.WHITE, Color.BLACK), this.getX(), this.getY());
			}
			// SI LO MATAMOS, SE MUERE
			if (this.pjObjetivo.SALUD <= 0) {
				this.pjObjetivo.SALUD = 0;
				this.pjObjetivo.muerte();
				this.pjObjetivo = null; // No tenemos objetivo
				this.caminoObjetivo = null; // Tenemos que buscar un camino
				this.estado = 1;// vamos a caminar
			}
		}

		return res;
	}

	/**
	 * @param game
	 */
	protected void accionCaminar(WarInTheDeepGame game) {
		// Si tenemos un camino, tomamos el paso...
		if (this.caminoObjetivo != null) {
			// Si hay personaje objetivo
			if (this.getPjObjetivo() != null) {
				// Si ya estoy en rango de un enemigo, voy a atacar
				Punto origen = new Punto(this.getX(), this.getY());
				Punto destino = new Punto(this.getPjObjetivo().getX(), this.getPjObjetivo().getY());
				if (AyudaDistancia.distanciaEntre(origen, destino) > this.MOD_RANGO) {
					for (int i = 0; i < this.MOD_VELOCIDAD; i++) {
						int nuevoX = this.caminoObjetivo.getX(i + 1);// Indice 0 es el punto
																		// actual, 1 es el
																		// primer paso
						int nuevoY = this.caminoObjetivo.getY(i + 1);// Indice 0 es el punto
																		// actual, 1 es el
																		// primer paso

						// Nos movemos
						this.setLocation(nuevoX, nuevoY);

					}
				} else {
					// Si estamos en rango, pero sobre algo
					if (game.getObjectsAt(this.getX(), this.getY(), null).size() > 1) {
						// Buscamos un punto libre
						Punto puntoNuevo = game.obtenerPuntoVacioCerca(this.getX(), this.getY());
						// Nos corremos a un lugar vacio
						this.setLocation(puntoNuevo.getX(), puntoNuevo.getY());
					}
				}
				// Giramos hacia el personaje
				this.turnTowards(destino.getX(), destino.getY());
			}
		}
	}

	/**
	 * Sucede la muerte de este Personaje
	 */
	protected void muerte() {
		this.setRotation(0);
		this.setImage("tumba2.png");
		this.estado = 0;
		this.pjObjetivo = null;
		this.caminoObjetivo = null;
	}

	/**
	 * @return obtiene la propiedad pjObjetivo
	 */
	public Personaje getPjObjetivo() {
		return this.pjObjetivo;
	}

	/**
	 * @param objetivo el Personaje objetivo
	 */
	public void setPjObjetivo(Personaje objetivo) {
		this.pjObjetivo = objetivo;
	}

	/**
	 * @param caminoObjetivo carga la propiedad caminoObjetivo
	 */
	public void setCaminoObjetivo(Camino caminoObjetivo) {
		this.caminoObjetivo = caminoObjetivo;
	}

	/**
	 * Obtiene una lista de enemigos que estan vivos en el escenario
	 * 
	 * @return una lista de Personajes objetivos vivos
	 */
	protected abstract List<Personaje> obtenerEnemigosVivos();

	/**
	 * Obtiene los enemigos vivos que estan en el rango de este personaje
	 * 
	 * @return una lista de Personajes objetivos vivos en rango
	 */
	protected abstract List<Personaje> obtenerEnemigosEnRango();

	/**
	 * Comprueba si un Personaje es amigo o no. Este metodo debe ser implementado por cada bando
	 * 
	 * @param amigo el Personaje que supuestamente es amigo
	 * @return TRUE si es del mismo bando, FALSE en cualquier otro caso
	 */
	protected abstract boolean esAmigo(Personaje amigo);

	/**
	 * Comprueba si un Personaje es enemigo o no. Este metodo debe ser implementado por cada bando
	 * 
	 * @param enemigo el Personaje que supuestamente es enemigo
	 * @return TRUE si es del otro bando, FALSE en cualquier otro caso
	 */
	protected abstract boolean esEnemigo(Personaje enemigo);
}
